from .codex import Codex, NPCodex
from .bedtools import bedtools
from .fetch import fetch_files
from .labeler import Labeler
# from .ranges import LabeledRange, LabeledRanges

name = 'ntai'

major = 0
minor = 0
patch = 16
version = '{}.{}.{}'.format(major, minor, patch)

description = 'Nucleotide A.I.'
